from atopile.parse import parser, parse_text_as_file

vdiv_model = """
component Resistor:
    value = "UNKNOWN"
    pin 1
    pin 2

module VDiv:
    v_out = "UNKNOWN"
    v_in = "UNKNOWN"
    q_current = "UNKNOWN"
    r_total = "UNKNOWN"
    ratio = "UNKNOWN"

    equations = "v_out == v_in * (r_top.value / (r_bottom.value + r_top.value)); q_current == v_in / (r_bottom.value + r_bottom.value); r_total == r_bottom.value + r_bottom.value; ratio == r_bottom.value / (r_bottom.value + r_bottom.value)"

    signal top
    signal out
    signal bottom

    r_top = new Resistor
    r_bottom = new Resistor

module Root:
    vdiv = new VDiv

    vdiv.v_in = 10
    vdiv.v_out = 5
    vdiv.q_current = 1
"""

parser.cache["dummy"] = parse_text_as_file(vdiv_model)

ROOT = "dummy::Root"



import pytest

from atopile.address import (
    add_instance,
    add_instances,
    get_file,
    get_instance_section,
    get_parent_instance_addr,
    get_instance_names,
)


def test_get_file():
    # Test with a file address
    assert get_file("path/to/file.txt") == "path/to/file.txt"

    # Test with a file address containing special characters
    assert (
        get_file("path/to/file-with-special-chars.txt")
        == "path/to/file-with-special-chars.txt"
    )

    # Test with a file address containing a Windows drive letter
    assert get_file("C:/path/to/file.txt") == "C:/path/to/file.txt"

    # Test with an empty file address
    assert get_file("") == ""

    assert get_file("path/to/file.txt:a:b.c") == "path/to/file.txt"

    # Test with a file address containing special characters
    assert (
        get_file("path/to/file-with-special-chars.txt:a:b.c")
        == "path/to/file-with-special-chars.txt"
    )

    # Test with a file address containing a Windows drive letter
    assert get_file("C:/path/to/file.txt:a:b.c") == "C:/path/to/file.txt"


def test_get_instance_section():
    # Test with an address containing an instance section
    assert get_instance_section("path/to/file.txt:a:b.c") is None

    # Test with an address not containing an instance section
    assert get_instance_section("path/to/file.txt") is None

    # Test with an empty address
    assert get_instance_section("") is None

    # Test with an address containing a Windows drive letter
    assert get_instance_section("C:/path/to/file.txt:a::b.c") == "b.c"

    # Test with an address containing a Windows drive letter
    assert get_instance_section("C:/path/to/file-with-special-chars.txt:a::b.c") == "b.c"


def test_add_instance():
    assert add_instance("//a:b", "c") == "//a:b::c"

    with pytest.raises(Exception):
        add_instance("//a", "c")


def test_add_instances():
    assert add_instances("//a:b::", ["c", "d", "e"]) == "//a:b::c.d.e"
    assert add_instances("//a:b", ["c", "d", "e"]) == "//a:b::c.d.e"
    assert add_instances("//a:b", []) == "//a:b"


def test_get_parent():
    assert get_parent_instance_addr("//a:b::c") == "//a:b"
    assert get_parent_instance_addr("//a:b") is None
    assert get_parent_instance_addr("//a:b::") == "//a:b"
    assert get_parent_instance_addr("//a:b::c.d") == "//a:b::c"
    assert get_parent_instance_addr("//a:b::c.d.e") == "//a:b::c.d"


def test_get_instances():
    assert get_instance_names("//a:b::c.d.e") == ["c", "d", "e"]
    assert get_instance_names("//a:b") == []
    assert get_instance_names("//a:b::") == []
    assert get_instance_names("//a:b::c") == ["c"]
    assert get_instance_names("//a:b::c.d") == ["c", "d"]



from atopile import instance_methods
from unittest.mock import MagicMock


def test_common_children():
    a = MagicMock()
    a.children = {
        "a": MagicMock(children={"a": MagicMock(children={})}),
        "b": MagicMock(children={}),
        "c": MagicMock(children={}),
    }

    children = list(instance_methods._common_children(a, a))
    r_aa, r_a, r_b, r_c = children

    assert r_a == (a.children["a"], a.children["a"])
    assert r_aa == (a.children["a"].children["a"], a.children["a"].children["a"])
    assert r_b == (a.children["b"], a.children["b"])
    assert r_c == (a.children["c"], a.children["c"])

    b = MagicMock(children={"a": MagicMock(children={})})
    assert list(instance_methods._common_children(a, b)) == [(a.children["a"], b.children["a"])]
